@@ -340,11 +340,9 @@ class SourcePackage(ABC):
     def _archive_servers(self):
         "Generator for mirror and master servers"
         # Always provide the mirrors first
-        for server in self.mirrors:
-            yield server
+        yield from self.mirrors
         # Don't repeat servers that are in both mirrors and masters
-        for server in set(self.masters) - set(self.mirrors):
-            yield server
+        yield from set(self.masters) - set(self.mirrors)
 
     def _source_urls(self, name):
         "Generator of sources for name"
@@ -635,8 +633,7 @@ class DebianSourcePackage(SourcePackage):
 
     def _source_urls(self, name):
         "Generator of sources for name"
-        for url in super()._source_urls(name):
-            yield url
+        yield from super()._source_urls(name)
         if name in self.snapshot_files:
             yield self.snapshot_files[name]
 
@@ -734,6 +731,7 @@ class PersonalPackageArchiveSourcePackage(UbuntuSourcePackage):
 
 class UbuntuCloudArchiveSourcePackage(PersonalPackageArchiveSourcePackage):
     "Download / unpack an Ubuntu Cloud Archive source package"
+
     TEAM = "ubuntu-cloud-archive"
     PROJECT = "cloud-archive"
     VALID_POCKETS = ["updates", "proposed", "staging"]
@@ -930,8 +928,8 @@ class UbuntuCloudArchiveSourcePackage(PersonalPackageArchiveSourcePackage):
 
 
 class _WebJSON:
-    def getHostUrl(self):  # pylint: disable=no-self-use
-        raise Exception("Not implemented")
+    def getHostUrl(self):
+        raise NotImplementedError(f"{self.__class__.__name__}.getHostUrl() is not implemented")
 
     def load(self, path=""):
         reader = codecs.getreader("utf-8")
