@@ -2,5 +2,5 @@
 #  ubuntu-dev-tools Launchpad Python modules.
 #
 
-service = 'production'
-api_version = '1.0'
+SERVICE = "production"
+API_VERSION = "devel"
