@@ -26,6 +26,7 @@ import logging
 import os
 import re
 from copy import copy
+from typing import Any
 from urllib.error import URLError
 from urllib.parse import urlparse
 
@@ -139,7 +140,7 @@ class BaseWrapper(metaclass=MetaWrapper):
     A base class from which other wrapper classes are derived.
     """
 
-    resource_type: str = None  # it's a base class after all
+    resource_type: str | tuple[str, str] = ""  # it's a base class after all
 
     def __new__(cls, data):
         if isinstance(data, str) and data.startswith(str(Launchpad._root_uri)):
@@ -290,9 +291,8 @@ class Distribution(BaseWrapper):
         Returns a list of all DistroSeries objects.
         """
         if not self._have_all_series:
-            for series in Launchpad.load(self.series_collection_link).entries:
-                series_link = DistroSeries(series["self_link"])
-                self._cache_series(series_link)
+            for series in self.series:
+                self._cache_series(DistroSeries(series))
             self._have_all_series = True
 
         allseries = filter(lambda s: s.active, self._series.values())
@@ -668,20 +668,19 @@ class Archive(BaseWrapper):
                     rversion = getattr(record, "binary_package_version", None)
                 else:
                     rversion = getattr(record, "source_package_version", None)
-                skipmsg = f"Skipping version {rversion}: "
 
                 if record.pocket not in pockets:
                     err_msg = f"pocket {record.pocket} not in ({','.join(pockets)})"
-                    Logger.debug(skipmsg + err_msg)
+                    Logger.debug("Skipping version %s: %s", rversion, err_msg)
                     continue
                 if record.status not in statuses:
                     err_msg = f"status {record.status} not in ({','.join(statuses)})"
-                    Logger.debug(skipmsg + err_msg)
+                    Logger.debug("Skipping version %s: %s", rversion, err_msg)
                     continue
                 release = wrapper(record)
                 if binary and archtag and archtag != release.arch:
                     err_msg = f"arch {release.arch} does not match requested arch {archtag}"
-                    Logger.debug(skipmsg + err_msg)
+                    Logger.debug("Skipping version %s: %s", rversion, err_msg)
                     continue
                 # results are ordered so first is latest
                 cache[index] = release
@@ -1406,10 +1405,7 @@ class PersonTeam(BaseWrapper, metaclass=MetaPersonTeam):
 
     def getPPAs(self):
         if self._ppas is None:
-            ppas = [
-                Archive(ppa["self_link"])
-                for ppa in Launchpad.load(self._lpobject.ppas_collection_link).entries
-            ]
+            ppas = [Archive(ppa) for ppa in self._lpobject.ppas]
             self._ppas = {ppa.name: ppa for ppa in ppas}
         return self._ppas
 
@@ -1434,10 +1430,7 @@ class Project(BaseWrapper):
         The list will be sorted by date_created, in descending order.
         """
         if not self._series:
-            series = [
-                ProjectSeries(s["self_link"])
-                for s in Launchpad.load(self._lpobject.series_collection_link).entries
-            ]
+            series = [ProjectSeries(s) for s in self._lpobject.series]
             self._series = sorted(series, key=lambda s: s.date_created, reverse=True)
         return self._series.copy()
 
@@ -1509,7 +1502,7 @@ class Packageset(BaseWrapper):  # pylint: disable=too-few-public-methods
 
     resource_type = "packageset"
     _lp_packagesets = None
-    _source_sets = {}
+    _source_sets: dict[tuple[str, str | None, bool], Any] = {}
 
     @classmethod
     def setsIncludingSource(cls, sourcepackagename, distroseries=None, direct_inclusion=False):
