@@ -436,7 +436,7 @@ class PullPkg:
         if options["upload_queue"]:
             # upload queue API is different/simpler
             self.pull_upload_queue(  # pylint: disable=missing-kwoa
-                pull, arch=options["arch"], download_only=options["download_only"], **params
+                pull, download_only=options["download_only"], **params
             )
             return
 
@@ -471,6 +471,7 @@ class PullPkg:
                     uri,
                 )
 
+                vcscmd = ""
                 if vcs == "Bazaar":
                     vcscmd = " $ bzr branch " + uri
                 elif vcs == "Git":
