@@ -15,30 +15,33 @@
 # ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 # OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
-from __future__ import print_function
-
+import logging
 import os
+import pathlib
 import re
+import subprocess
 import sys
 
 import debian.changelog
 import debian.deb822
 
-from ubuntutools import subprocess
-from ubuntutools.logger import Logger
 from ubuntutools.question import Question, YesNoQuestion
+from ubuntutools.sponsor_patch.question import (
+    ask_for_ignoring_or_fixing,
+    ask_for_manual_fixing,
+    user_abort,
+)
 
-from ubuntutools.sponsor_patch.question import (ask_for_ignoring_or_fixing,
-                                                ask_for_manual_fixing,
-                                                user_abort)
+Logger = logging.getLogger(__name__)
 
 
 def _get_series(launchpad):
     """Returns a tuple with the development and list of supported series."""
-    ubuntu = launchpad.distributions['ubuntu']
+    ubuntu = launchpad.distributions["ubuntu"]
     devel_series = ubuntu.current_series.name
-    supported_series = [series.name for series in ubuntu.series
-                        if series.active and series.name != devel_series]
+    supported_series = [
+        series.name for series in ubuntu.series if series.active and series.name != devel_series
+    ]
     return (devel_series, supported_series)
 
 
@@ -49,14 +52,14 @@ def strip_epoch(version):
     return "1.1.3-1".
     """
 
-    parts = version.full_version.split(':')
+    parts = version.full_version.split(":")
     if len(parts) > 1:
         del parts[0]
-    version_without_epoch = ':'.join(parts)
+    version_without_epoch = ":".join(parts)
     return version_without_epoch
 
 
-class SourcePackage(object):
+class SourcePackage:
     """This class represents a source package."""
 
     def __init__(self, package, builder, workdir, branch):
@@ -74,11 +77,10 @@ class SourcePackage(object):
         if upload == "ubuntu":
             self._print_logs()
             question = Question(["yes", "edit", "no"])
-            answer = question.ask("Do you want to acknowledge the sync request",
-                                  "no")
+            answer = question.ask("Do you want to acknowledge the sync request", "no")
             if answer == "edit":
                 return False
-            elif answer == "no":
+            if answer == "no":
                 user_abort()
 
             bug = task.bug
@@ -86,37 +88,36 @@ class SourcePackage(object):
             if task.importance == "Undecided":
                 task.importance = "Wishlist"
             task.lp_save()
-            Logger.info("Set bug #%i status to Confirmed.", bug.id)
+            Logger.debug("Set bug #%i status to Confirmed.", bug.id)
 
             msg = "Sync request ACK'd."
             if self._build_log:
-                msg = ("%s %s builds on %s. " + msg) % \
-                      (self._package, self._version,
-                       self._builder.get_architecture())
+                msg = (
+                    f"{self._package} {self._version} builds"
+                    f" on {self._builder.get_architecture()}. {msg}"
+                )
             bug.newMessage(content=msg, subject="sponsor-patch")
-            Logger.info("Acknowledged sync request bug #%i.", bug.id)
+            Logger.debug("Acknowledged sync request bug #%i.", bug.id)
 
-            bug.subscribe(person=launchpad.people['ubuntu-archive'])
-            Logger.info("Subscribed ubuntu-archive to bug #%i.", bug.id)
+            bug.subscribe(person=launchpad.people["ubuntu-archive"])
+            Logger.debug("Subscribed ubuntu-archive to bug #%i.", bug.id)
 
             bug.subscribe(person=launchpad.me)
-            Logger.info("Subscribed me to bug #%i.", bug.id)
+            Logger.debug("Subscribed me to bug #%i.", bug.id)
 
-            sponsorsteam = launchpad.people['ubuntu-sponsors']
+            sponsorsteam = launchpad.people["ubuntu-sponsors"]
             for sub in bug.subscriptions:
                 if sub.person == sponsorsteam and sub.canBeUnsubscribedByUser():
-                    bug.unsubscribe(person=launchpad.people['ubuntu-sponsors'])
-                    Logger.info("Unsubscribed ubuntu-sponsors from bug #%i.",
-                                bug.id)
+                    bug.unsubscribe(person=launchpad.people["ubuntu-sponsors"])
+                    Logger.debug("Unsubscribed ubuntu-sponsors from bug #%i.", bug.id)
                 elif sub.person == sponsorsteam:
-                    Logger.info("Couldn't unsubscribe ubuntu-sponsors from "
-                                "bug #%i.", bug.id)
+                    Logger.debug("Couldn't unsubscribe ubuntu-sponsors from bug #%i.", bug.id)
 
-            Logger.normal("Successfully acknowledged sync request bug #%i.",
-                          bug.id)
+            Logger.info("Successfully acknowledged sync request bug #%i.", bug.id)
         else:
-            Logger.error("Sync requests can only be acknowledged when the "
-                         "upload target is Ubuntu.")
+            Logger.error(
+                "Sync requests can only be acknowledged when the upload target is Ubuntu."
+            )
             sys.exit(1)
         return True
 
@@ -135,34 +136,35 @@ class SourcePackage(object):
             else:
                 target = upload
             question = Question(["yes", "edit", "no"])
-            answer = question.ask("Do you want to upload the package to %s" % target, "no")
+            answer = question.ask(f"Do you want to upload the package to {target}", "no")
             if answer == "edit":
                 return False
-            elif answer == "no":
+            if answer == "no":
                 user_abort()
             cmd = ["dput", "--force", upload, self._changes_file]
-            Logger.command(cmd)
+            Logger.debug(" ".join(cmd))
             if subprocess.call(cmd) != 0:
-                Logger.error("Upload of %s to %s failed." %
-                             (os.path.basename(self._changes_file), upload))
+                Logger.error(
+                    "Upload of %s to %s failed.", os.path.basename(self._changes_file), upload
+                )
                 sys.exit(1)
 
             # Push the branch if the package is uploaded to the Ubuntu archive.
             if upload == "ubuntu" and self._branch:
-                cmd = ['debcommit']
-                Logger.command(cmd)
+                cmd = ["debcommit"]
+                Logger.debug(" ".join(cmd))
                 if subprocess.call(cmd) != 0:
-                    Logger.error('Bzr commit failed.')
+                    Logger.error("Bzr commit failed.")
                     sys.exit(1)
-                cmd = ['bzr', 'mark-uploaded']
-                Logger.command(cmd)
+                cmd = ["bzr", "mark-uploaded"]
+                Logger.debug(" ".join(cmd))
                 if subprocess.call(cmd) != 0:
-                    Logger.error('Bzr tagging failed.')
+                    Logger.error("Bzr tagging failed.")
                     sys.exit(1)
-                cmd = ['bzr', 'push', ':parent']
-                Logger.command(cmd)
+                cmd = ["bzr", "push", ":parent"]
+                Logger.debug(" ".join(cmd))
                 if subprocess.call(cmd) != 0:
-                    Logger.error('Bzr push failed.')
+                    Logger.error("Bzr push failed.")
                     sys.exit(1)
         return True
 
@@ -175,8 +177,10 @@ class SourcePackage(object):
 
         if dist is None:
             dist = re.sub("-.*$", "", self._changelog.distributions)
-        build_name = "{}_{}_{}.build".format(self._package, strip_epoch(self._version),
-                                             self._builder.get_architecture())
+        build_name = (
+            f"{self._package}_{strip_epoch(self._version)}"
+            f"_{self._builder.get_architecture()}.build"
+        )
         self._build_log = os.path.join(self._buildresult, build_name)
 
         successful_built = False
@@ -191,20 +195,18 @@ class SourcePackage(object):
                 update = False
 
             # build package
-            result = self._builder.build(self._dsc_file, dist,
-                                         self._buildresult)
+            result = self._builder.build(self._dsc_file, dist, self._buildresult)
             if result != 0:
                 question = Question(["yes", "update", "retry", "no"])
                 answer = question.ask("Do you want to resolve this issue manually", "yes")
                 if answer == "yes":
                     break
-                elif answer == "update":
+                if answer == "update":
                     update = True
                     continue
-                elif answer == "retry":
+                if answer == "retry":
                     continue
-                else:
-                    user_abort()
+                user_abort()
             successful_built = True
         if not successful_built:
             # We want to do a manual fix if the build failed.
@@ -224,13 +226,14 @@ class SourcePackage(object):
         """
 
         if self._branch:
-            cmd = ['bzr', 'builddeb', '--builder=debuild', '-S',
-                   '--', '--no-lintian', '-nc']
+            cmd = ["bzr", "builddeb", "--builder=debuild", "-S", "--", "--no-lintian", "-nc"]
         else:
-            cmd = ['debuild', '--no-lintian', '-nc', '-S']
+            cmd = ["debuild", "--no-lintian", "-nc", "-S"]
         cmd.append("-v" + previous_version.full_version)
-        if previous_version.upstream_version == \
-           self._changelog.upstream_version and upload == "ubuntu":
+        if (
+            previous_version.upstream_version == self._changelog.upstream_version
+            and upload == "ubuntu"
+        ):
             # FIXME: Add proper check that catches cases like changed
             # compression (.tar.gz -> tar.bz2) and multiple orig source tarballs
             cmd.append("-sd")
@@ -239,9 +242,9 @@ class SourcePackage(object):
         if keyid is not None:
             cmd += ["-k" + keyid]
         env = os.environ
-        if upload == 'ubuntu':
-            env['DEB_VENDOR'] = 'Ubuntu'
-        Logger.command(cmd)
+        if upload == "ubuntu":
+            env["DEB_VENDOR"] = "Ubuntu"
+        Logger.debug(" ".join(cmd))
         if subprocess.call(cmd, env=env) != 0:
             Logger.error("Failed to build source tarball.")
             # TODO: Add a "retry" option
@@ -252,8 +255,9 @@ class SourcePackage(object):
     @property
     def _changes_file(self):
         """Returns the file name of the .changes file."""
-        return os.path.join(self._workdir, "{}_{}_source.changes"
-                            .format(self._package, strip_epoch(self._version)))
+        return os.path.join(
+            self._workdir, f"{self._package}_{strip_epoch(self._version)}_source.changes"
+        )
 
     def check_target(self, upload, launchpad):
         """Make sure that the target is correct.
@@ -265,18 +269,24 @@ class SourcePackage(object):
         (devel_series, supported_series) = _get_series(launchpad)
 
         if upload == "ubuntu":
-            allowed = supported_series + \
-                      [s + "-proposed" for s in supported_series] + \
-                      [devel_series]
+            allowed = (
+                supported_series + [s + "-proposed" for s in supported_series] + [devel_series]
+            )
             if self._changelog.distributions not in allowed:
-                Logger.error("%s is not an allowed series. It needs to be one of %s." %
-                             (self._changelog.distributions, ", ".join(allowed)))
+                Logger.error(
+                    "%s is not an allowed series. It needs to be one of %s.",
+                    self._changelog.distributions,
+                    ", ".join(allowed),
+                )
                 return ask_for_ignoring_or_fixing()
         elif upload and upload.startswith("ppa/"):
             allowed = supported_series + [devel_series]
             if self._changelog.distributions not in allowed:
-                Logger.error("%s is not an allowed series. It needs to be one of %s." %
-                             (self._changelog.distributions, ", ".join(allowed)))
+                Logger.error(
+                    "%s is not an allowed series. It needs to be one of %s.",
+                    self._changelog.distributions,
+                    ", ".join(allowed),
+                )
                 return ask_for_ignoring_or_fixing()
         return True
 
@@ -288,18 +298,21 @@ class SourcePackage(object):
         """
 
         if self._version <= previous_version:
-            Logger.error("The version %s is not greater than the already "
-                         "available %s.", self._version, previous_version)
+            Logger.error(
+                "The version %s is not greater than the already available %s.",
+                self._version,
+                previous_version,
+            )
             return ask_for_ignoring_or_fixing()
         return True
 
     def check_sync_request_version(self, bug_number, task):
         """Check if the downloaded version of the package is mentioned in the
-           bug title."""
+        bug title."""
 
         if not task.title_contains(self._version):
-            print("Bug #%i title: %s" % (bug_number, task.get_bug_title()))
-            msg = "Is %s %s the version that should be synced" % (self._package, self._version)
+            print(f"Bug #{bug_number} title: {task.get_bug_title()}")
+            msg = f"Is {self._package} {self._version} the version that should be synced"
             answer = YesNoQuestion().ask(msg, "no")
             if answer == "no":
                 user_abort()
@@ -307,33 +320,27 @@ class SourcePackage(object):
     @property
     def _debdiff_filename(self):
         """Returns the file name of the .debdiff file."""
-        debdiff_name = "{}_{}.debdiff".format(self._package, strip_epoch(self._version))
+        debdiff_name = f"{self._package}_{strip_epoch(self._version)}.debdiff"
         return os.path.join(self._workdir, debdiff_name)
 
     @property
     def _dsc_file(self):
         """Returns the file name of the .dsc file."""
-        return os.path.join(self._workdir, "{}_{}.dsc".format(self._package,
-                                                              strip_epoch(self._version)))
+        return os.path.join(self._workdir, f"{self._package}_{strip_epoch(self._version)}.dsc")
 
     def generate_debdiff(self, dsc_file):
         """Generates a debdiff between the given .dsc file and this source
-           package."""
+        package."""
 
-        assert os.path.isfile(dsc_file), "%s does not exist." % (dsc_file)
-        assert os.path.isfile(self._dsc_file), "%s does not exist." % \
-                                               (self._dsc_file)
+        assert os.path.isfile(dsc_file), f"{dsc_file} does not exist."
+        assert os.path.isfile(self._dsc_file), f"{self._dsc_file} does not exist."
         cmd = ["debdiff", dsc_file, self._dsc_file]
-        if not Logger.verbose:
+        if not Logger.isEnabledFor(logging.DEBUG):
             cmd.insert(1, "-q")
-        Logger.command(cmd + [">", self._debdiff_filename])
-        process = subprocess.Popen(cmd, stdout=subprocess.PIPE)
-        debdiff = process.communicate()[0]
-
-        # write debdiff file
-        debdiff_file = open(self._debdiff_filename, "w")
-        debdiff_file.writelines(debdiff)
-        debdiff_file.close()
+        Logger.debug("%s > %s", " ".join(cmd), self._debdiff_filename)
+        with open(self._debdiff_filename, "w", encoding="utf-8") as debdiff_file:
+            debdiff = subprocess.run(cmd, check=False, stdout=debdiff_file)
+            assert debdiff.returncode in (0, 1)
 
     def is_fixed(self, lp_bug):
         """Make sure that the given Launchpad bug is closed.
@@ -342,8 +349,8 @@ class SourcePackage(object):
         change something.
         """
 
-        assert os.path.isfile(self._changes_file), "%s does not exist." % (self._changes_file)
-        changes = debian.deb822.Changes(open(self._changes_file))
+        assert os.path.isfile(self._changes_file), f"{self._changes_file} does not exist."
+        changes = debian.deb822.Changes(open(self._changes_file, encoding="utf-8"))
         fixed_bugs = []
         if "Launchpad-Bugs-Fixed" in changes:
             fixed_bugs = changes["Launchpad-Bugs-Fixed"].split(" ")
@@ -355,7 +362,7 @@ class SourcePackage(object):
             lp_bug = lp_bug.duplicate_of
 
         if lp_bug.id not in fixed_bugs:
-            Logger.error("Launchpad bug #%i is not closed by new version." % (lp_bug.id))
+            Logger.error("Launchpad bug #%i is not closed by new version.", lp_bug.id)
             return ask_for_ignoring_or_fixing()
         return True
 
@@ -363,7 +370,7 @@ class SourcePackage(object):
         """Print things that should be checked before uploading a package."""
 
         lintian_filename = self._run_lintian()
-        print("\nPlease check %s %s carefully:" % (self._package, self._version))
+        print(f"\nPlease check {self._package} {self._version} carefully:")
         if os.path.isfile(self._debdiff_filename):
             print("file://" + self._debdiff_filename)
         print("file://" + lintian_filename)
@@ -380,8 +387,9 @@ class SourcePackage(object):
         # Check the changelog
         self._changelog = debian.changelog.Changelog()
         try:
-            self._changelog.parse_changelog(open("debian/changelog"),
-                                            max_blocks=1, strict=True)
+            self._changelog.parse_changelog(
+                open("debian/changelog", encoding="utf-8"), max_blocks=1, strict=True
+            )
         except debian.changelog.ChangelogParseError as error:
             Logger.error("The changelog entry doesn't validate: %s", str(error))
             ask_for_manual_fixing()
@@ -391,61 +399,62 @@ class SourcePackage(object):
         try:
             self._version = self._changelog.get_version()
         except IndexError:
-            Logger.error("Debian package version could not be determined. "
-                         "debian/changelog is probably malformed.")
+            Logger.error(
+                "Debian package version could not be determined. "
+                "debian/changelog is probably malformed."
+            )
             ask_for_manual_fixing()
             return False
 
         return True
 
-    def _run_lintian(self):
+    def _run_lintian(self) -> str:
         """Runs lintian on either the source or binary changes file.
 
         Returns the filename of the created lintian output file.
         """
 
         # Determine whether to use the source or binary build for lintian
+        package_and_version = f"{self._package}_{strip_epoch(self._version)}"
         if self._build_log:
-            build_changes = self._package + "_" + strip_epoch(self._version) + \
-                           "_" + self._builder.get_architecture() + ".changes"
+            build_changes = f"{package_and_version}_{self._builder.get_architecture()}.changes"
             changes_for_lintian = os.path.join(self._buildresult, build_changes)
         else:
             changes_for_lintian = self._changes_file
 
         # Check lintian
-        assert os.path.isfile(changes_for_lintian), "%s does not exist." % \
-                                                    (changes_for_lintian)
-        cmd = ["lintian", "-IE", "--pedantic", "-q", "--profile", "ubuntu",
-               changes_for_lintian]
-        lintian_filename = os.path.join(self._workdir,
-                                        self._package + "_" +
-                                        strip_epoch(self._version) + ".lintian")
-        Logger.command(cmd + [">", lintian_filename])
-        process = subprocess.Popen(cmd, stdout=subprocess.PIPE)
-        report = process.communicate()[0]
+        assert os.path.isfile(changes_for_lintian), f"{changes_for_lintian} does not exist."
+        cmd = ["lintian", "-IE", "--pedantic", "-q", "--profile", "ubuntu", changes_for_lintian]
+        lintian_file = pathlib.Path(self._workdir) / f"{package_and_version}.lintian"
+        Logger.debug("%s > %s", " ".join(cmd), lintian_file)
+        with lintian_file.open("wb") as outfile:
+            subprocess.run(cmd, stdout=outfile, check=True)
 
-        # write lintian report file
-        lintian_file = open(lintian_filename, "w")
-        lintian_file.writelines(report)
-        lintian_file.close()
-
-        return lintian_filename
+        return str(lintian_file)
 
     def sync(self, upload, series, bug_number, requester):
         """Does a sync of the source package."""
 
         if upload == "ubuntu":
-            cmd = ["syncpackage", self._package, "-b", str(bug_number), "-f",
-                   "-s", requester, "-V", str(self._version),
-                   "-d", series]
-            Logger.command(cmd)
+            cmd = [
+                "syncpackage",
+                self._package,
+                "-b",
+                str(bug_number),
+                "-f",
+                "-s",
+                requester,
+                "-V",
+                str(self._version),
+                "-d",
+                series,
+            ]
+            Logger.debug(" ".join(cmd))
             if subprocess.call(cmd) != 0:
-                Logger.error("Syncing of %s %s failed.", self._package,
-                             str(self._version))
+                Logger.error("Syncing of %s %s failed.", self._package, str(self._version))
                 sys.exit(1)
         else:
             # FIXME: Support this use case!
-            Logger.error("Uploading a synced package other than to Ubuntu "
-                         "is not supported yet!")
+            Logger.error("Uploading a synced package other than to Ubuntu is not supported yet!")
             sys.exit(1)
         return True
